@@ -18,17 +18,17 @@ prefer) for the classes/datatypes that are used in iCalendar:
 
 ###########################################################################
 
-iCalendar properties has values. The values are strongly typed. This module
-defines these types, calling val.to_ical() on them, Will render them as defined
+iCalendar properties have values. The values are strongly typed. This module
+defines these types, calling val.to_ical() on them will render them as defined
 in rfc2445.
 
 If you pass any of these classes a Python primitive, you will have an object
 that can render itself as iCalendar formatted date.
 
-Property Value Data Types starts with a 'v'. they all have an to_ical() and
+Property Value Data Types start with a 'v'. they all have an to_ical() and
 from_ical() method. The to_ical() method generates a text string in the
 iCalendar format. The from_ical() method can parse this format and return a
-primitive Python datatype. So it should allways be true that:
+primitive Python datatype. So it should always be true that:
 
     x == vDataType.from_ical(VDataType(x).to_ical())
 
@@ -55,6 +55,8 @@ from icalendar.parser import unescape_char
 from icalendar.parser_tools import DEFAULT_ENCODING
 from icalendar.parser_tools import SEQUENCE_TYPES
 from icalendar.parser_tools import to_unicode
+from icalendar.timezone_cache import _timezone_cache
+from icalendar.windows_to_olson import WINDOWS_TO_OLSON
 
 import base64
 import binascii
@@ -69,8 +71,8 @@ DATETIME_PART = '(?:%s)?(?:%s)?' % (DATE_PART, TIME_PART)
 WEEKS_PART = r'(\d+)W'
 DURATION_REGEX = re.compile(r'([-+]?)P(?:%s|%s)$'
                             % (WEEKS_PART, DATETIME_PART))
-WEEKDAY_RULE = re.compile('(?P<signal>[+-]?)(?P<relative>[\d]?)'
-                          '(?P<weekday>[\w]{2})$')
+WEEKDAY_RULE = re.compile(r'(?P<signal>[+-]?)(?P<relative>[\d]?)'
+                          r'(?P<weekday>[\w]{2})$')
 
 
 ####################################################
@@ -272,14 +274,17 @@ class vDDDTypes(object):
     So this is practical.
     """
     def __init__(self, dt):
-        if not isinstance(dt, (datetime, date, timedelta, time)):
-            raise ValueError('You must use datetime, date, timedelta or time')
+        if not isinstance(dt, (datetime, date, timedelta, time, tuple)):
+            raise ValueError('You must use datetime, date, timedelta, '
+                             'time or tuple (for periods)')
         if isinstance(dt, datetime):
-            self.params = Parameters(dict(value='DATE-TIME'))
+            self.params = Parameters({'value': 'DATE-TIME'})
         elif isinstance(dt, date):
-            self.params = Parameters(dict(value='DATE'))
+            self.params = Parameters({'value': 'DATE'})
         elif isinstance(dt, time):
-            self.params = Parameters(dict(value='TIME'))
+            self.params = Parameters({'value': 'TIME'})
+        elif isinstance(dt, tuple):
+            self.params = Parameters({'value': 'PERIOD'})
 
         if (isinstance(dt, datetime) or isinstance(dt, time))\
                 and getattr(dt, 'tzinfo', False):
@@ -302,23 +307,31 @@ class vDDDTypes(object):
             return vDuration(dt).to_ical()
         elif isinstance(dt, time):
             return vTime(dt).to_ical()
+        elif isinstance(dt, tuple) and len(dt) == 2:
+            return vPeriod(dt).to_ical()
         else:
-            raise ValueError('Unknown date type')
+            raise ValueError('Unknown date type: {}'.format(type(dt)))
 
     @classmethod
     def from_ical(cls, ical, timezone=None):
         if isinstance(ical, cls):
             return ical.dt
         u = ical.upper()
-        if u.startswith('-P') or u.startswith('P'):
+        if u.startswith(('P', '-P', '+P')):
             return vDuration.from_ical(ical)
-        try:
+        if '/' in u:
+            return vPeriod.from_ical(ical)
+
+        if len(ical) in (15, 16):
             return vDatetime.from_ical(ical, timezone=timezone)
-        except ValueError:
-            try:
-                return vDate.from_ical(ical)
-            except ValueError:
-                return vTime.from_ical(ical)
+        elif len(ical) == 8:
+            return vDate.from_ical(ical)
+        elif len(ical) in (6, 7):
+            return vTime.from_ical(ical)
+        else:
+            raise ValueError(
+                "Expected datetime, date, or time, got: '%s'" % ical
+            )
 
 
 class vDate(object):
@@ -328,7 +341,7 @@ class vDate(object):
         if not isinstance(dt, date):
             raise ValueError('Value MUST be a date instance')
         self.dt = dt
-        self.params = Parameters(dict(value='DATE'))
+        self.params = Parameters({'value': 'DATE'})
 
     def to_ical(self):
         s = "%04d%02d%02d" % (self.dt.year, self.dt.month, self.dt.day)
@@ -387,7 +400,10 @@ class vDatetime(object):
             try:
                 tzinfo = pytz.timezone(timezone)
             except pytz.UnknownTimeZoneError:
-                pass
+                if timezone in WINDOWS_TO_OLSON:
+                    tzinfo = pytz.timezone(WINDOWS_TO_OLSON.get(timezone))
+                else:
+                    tzinfo = _timezone_cache.get(timezone, None)
 
         try:
             timetuple = (
@@ -534,7 +550,7 @@ class vPeriod(object):
             p = (self.start, self.duration)
         else:
             p = (self.start, self.end)
-        return 'vPeriod(%r)' % p
+        return 'vPeriod(%r)' % (p, )
 
 
 class vWeekday(compat.unicode_type):
@@ -554,7 +570,7 @@ class vWeekday(compat.unicode_type):
         sign = match['signal']
         weekday = match['weekday']
         relative = match['relative']
-        if not weekday in vWeekday.week_days or sign not in '+-':
+        if weekday not in vWeekday.week_days or sign not in '+-':
             raise ValueError('Expected weekday abbrevation, got: %s' % self)
         self.relative = relative and int(relative) or None
         self.params = Parameters()
@@ -588,7 +604,7 @@ class vFrequency(compat.unicode_type):
     def __new__(cls, value, encoding=DEFAULT_ENCODING):
         value = to_unicode(value, encoding=encoding)
         self = super(vFrequency, cls).__new__(cls, value)
-        if not self in vFrequency.frequencies:
+        if self not in vFrequency.frequencies:
             raise ValueError('Expected frequency, got: %s' % self)
         self.params = Parameters()
         return self
@@ -624,6 +640,7 @@ class vRecur(CaselessDict):
         'BYSECOND': vInt,
         'BYMINUTE': vInt,
         'BYHOUR': vInt,
+        'BYWEEKNO': vInt,
         'BYMONTHDAY': vInt,
         'BYYEARDAY': vInt,
         'BYMONTH': vInt,
@@ -665,7 +682,12 @@ class vRecur(CaselessDict):
         try:
             recur = cls()
             for pairs in ical.split(';'):
-                key, vals = pairs.split('=')
+                try:
+                    key, vals = pairs.split('=')
+                except ValueError:
+                    # E.g. incorrect trailing semicolon, like (issue #157):
+                    # FREQ=YEARLY;BYMONTH=11;BYDAY=1SU;
+                    continue
                 recur[key] = cls.parse_type(key, vals)
             return dict(recur)
         except:
@@ -706,7 +728,7 @@ class vTime(object):
             self.dt = args[0]
         else:
             self.dt = time(*args)
-        self.params = Parameters(dict(value='TIME'))
+        self.params = Parameters({'value': 'TIME'})
 
     def to_ical(self):
         return self.dt.strftime("%H%M%S")
@@ -774,6 +796,11 @@ class vUTCOffset(object):
     """Renders itself as a utc offset.
     """
 
+    ignore_exceptions = False   # if True, and we cannot parse this
+                                # component, we will silently ignore
+                                # it, rather than let the exception
+                                # propagate upwards
+
     def __init__(self, td):
         if not isinstance(td, timedelta):
             raise ValueError('Offset value MUST be a timedelta instance')
@@ -813,7 +840,7 @@ class vUTCOffset(object):
             offset = timedelta(hours=hours, minutes=minutes, seconds=seconds)
         except:
             raise ValueError('Expected utc offset, got: %s' % ical)
-        if offset >= timedelta(hours=24):
+        if not cls.ignore_exceptions and offset >= timedelta(hours=24):
             raise ValueError(
                 'Offset must be less than 24 hours, was %s' % ical)
         if sign == '-':
